# Screen & Relax : A fast solving procedure for the Elastic-Net problem by safe identification of the solution support.

This repository implements the Screen & Relax method solving the Elastic-Net problem. We introduced this method in the paper

> Théo Guyard, Cédric Herzet, Clément Elvira, *Screen & Relax: Accelerating the resolution of Elastic-net by safe identification of the solution support*

available [here](https://arxiv.org/pdf/2110.07281.pdf). This paper contains theoretical and numerical results that can be reproduced with this toolbox. If you encounter a bug or something unexpected, please let me know by raising an issue on the project page or by contacting me by [mail](mailto:theo.guyard@insa-rennes.fr).

## Requirements

The `srenet` (**S**creen & **R**elax method for the **E**lastic-**NET** problem) package works with Python 3.9+. Please refer to the [Python download page](https://www.python.org/downloads/) for install instructions.

**Dependencies** : [NumPy](http://www.numpy.org), [SciPy](https://www.scipy.org), [Matplotlib](http://matplotlib.org), [Pandas](https://pandas.pydata.org)

## Install instructions

1. Clone the repository
```bash
git clone https://gitlab.insa-rennes.fr/Theo.Guyard/screen-and-relax.git
```

2. Enter the project folder
```bash
cd screen-and-relax
```

3. Ensure that Python 3.9+ is installed
```bash
python -V
> Python 3.9.1 (or higher)
```

4. Install external dependencies
```bash
pip install --upgrade pip
pip install -r requirements.txt
```

5. Install the `srenet` package
```bash
pip install .
```

## ICASSP experiments

Experiments presented in the Screen & Relax paper submitted at [ICASSP 2022](https://2022.ieeeicassp.org) are located in the folder `screen-and-relax/exp/ICASSP/`. The two available experiments are 
* `convergence.py` : convergence of the four compared methods on a single problem instance
* `performance_profiles.py` : figure 1 of the "Screen & Relax" paper

To run experiments, make sure to be located at the project root and run experiments as follows.
```bash
cd screen-and-relax
python exp/ICASSP/convergence.py
python exp/ICASSP/performance_profiles.py
```

You can also edit the experimental setup directly in the script, just follow the comments documenting the code.

## Running demo examples

We also provide a notebook explaining how to use the `srenet` package. You can use it as following :
```bash
cd notebooks
jupyter notebook
```
You may have to install Jupyter. Please refer to the [Jupyter website](https://jupyter.org) for install instructions.

## Licence

This software is distributed under the [MIT Licence](https://mit-license.org).


## Cite this work

If you use this package for your own work, please consider citing it as :

**Paper is being reviewed, the citation will be available soon.**
