import numpy as np
import matplotlib.pyplot as plt
from srenet.data import sample_data
from srenet.projgrad import projgrad
from srenet.utils import ProjgradParams

m, n = 100, 300                 # Size of A
gen_dic = 'gaussian'            # Sampling method for A
gen_obs = 'gaussian'            # Sampling method for y
l1ratio = 0.1                   # l1-regularizer / max(A.T @ y)
l2ratio = 0.1                   # l2-regularizer / max(A.T @ y)

x0 = np.zeros(n)
A, y = sample_data(m, n, gen_dic, gen_obs)
lmax = np.max(A.T @ y)
l1 = l1ratio * lmax
l2 = l2ratio * lmax

x, mnt = projgrad(x0, A, y, l1, l2, ProjgradParams(verbosity=False,screen=False,relax=False))
x_s, mnt_s = projgrad(x0, A, y, l1, l2, ProjgradParams(verbosity=False,screen=True,relax=False))
x_r, mnt_r = projgrad(x0, A, y, l1, l2, ProjgradParams(verbosity=False,screen=False,relax=True))
x_sr, mnt_sr = projgrad(x0, A, y, l1, l2, ProjgradParams(verbosity=False,screen=True,relax=True))

res = {
    'aPG' : {'x': x,    'mnt': mnt},
    'aPGs': {'x': x_s,  'mnt': mnt_s},
    'aPGr': {'x': x_r,  'mnt': mnt_r},
    'S\&R' : {'x': x_sr, 'mnt': mnt_sr},
}

print("----------------")
print("Convergence test")
for k, v in res.items():
    print(k)
    print("  Stop crit       : {}".format(v['mnt'].stopcrit))
    print("  Last Gap        : {:.2e}".format(v['mnt'].gap[-1]))
    print("  Last Obj        : {:.2e}".format(v['mnt'].obj[-1]))
    print("  Iters           : {}".format(len(v['mnt'].gap)))
    print("  FLOPs           : {:.2e}".format(v['mnt'].flops[-1]))

plt.rcParams["font.size"] = 12
plt.rcParams["axes.labelsize"] = 12
plt.rcParams["axes.titlesize"] = 18
plt.rcParams["lines.linewidth"] = 2
plt.rcParams["lines.markersize"] = 5
plt.rcParams["xtick.labelsize"] = 12
plt.rcParams["ytick.labelsize"] = 12
plt.rcParams["text.usetex"] = True
plt.rcParams["font.family"] = "serif"
plt.rcParams["font.serif"] = ["Helvetica"]

fig, ax = plt.subplots(1,1,figsize=(4,3))
for k,v in res.items():
    ax.plot(v['mnt'].flops, v['mnt'].obj-v['mnt'].obj[-1], label=k, marker='.')
ax.set_xlabel(r"FLOPs")
ax.set_ylabel(r"$f(x)-f(x^{\star})$")
ax.set_xscale('log')
ax.set_yscale('log')
ax.grid(b=True, which='major', axis='y')
ax.legend()
plt.tight_layout()
plt.show()

print("----------------")